pro CORRSENT,LOG,image_type,PLOT=plot,extt=extt,xoffset=xoffset,lamp=lamp,nosp=nosp,out=out,nobgd=nobgd,nullsent=nullsent,old=old
;+
; NAME:
;	CORRSENT
; PURPOSE:
;	correction spectral sensitivity
; DESCRIPTION:
;	The function computes the absolute spectral distribution in reduced
;	2-dimensional array MPFS-spectra in erg/cm^2/sec/A.
;	Routine read vector spectral sensitivity
;	from disk in working directory with standard name 'sent.fts',
;	sky-substraction linearised spectra (filename='*-sky.fts', where
;	image_type='*')	and modified FITS header
;
; CALLING SEQUENCE:
;	CORRSENT, LOG,file_name,PLOT=plot
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOG = String scalarfile name of FITS-header from the LOG observation MPFS data
;	image_type= string scalar :( value 'obj' or 'test' )
;
; OUTPUTS:
;	Saved in working directory in file '*_abs.fts'
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	PLOT - if present  then plotted spectra central part lens array to display,
;	else save plot to POSTSCRIPT file in working directory with name '*.ps'
;       xoffset - lower and upper limits for spectrum fitting and extraction
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   DEF_WDIR,READ_FTS,SXPAR,OS_FAMILY
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-

if not(keyword_set(out)) then out='_abs'
on_error,0
message,'flux calibration '+image_type+'-spectra',/cont
w=3.
wdir=DEF_WDIR(LOG)

sent=READ_FTS(wdir+'sent.fts',h)

if not(keyword_set(xoffset)) then xoffset=[0.,0.]
if not(keyword_set(extt)) then extt='-sky'
spectr=READ_FTS(wdir+image_type+extt+'.fts',ho)             ;*6
a=size(spectr)
spectr=spectr(xoffset[0]:a[1]-xoffset[1]-1,*)
if a[2] gt 256 then spectr=spectr(*,a[2]-256:a[2]-1)
a=size(spectr)

N_x=sxpar(h,'NAXIS1')	         ;number spectral element
N_y=a[2]               ;number of spectra
N_xo=a[1]	         ;number spectral element
N_yo=a[2]               ;number of spectra

obj_red=dblarr(N_xo,N_yo)
param=[sxpar(h,'CRVAL1'),sxpar(h,'CDELT1'),N_x]
paramo=[sxpar(ho,'CRVAL1')+xoffset[0]*sxpar(ho,'CDELT1'),sxpar(ho,'CDELT1'),N_xo]

z=double(sxpar(h,'REDSHIFT'))
if not(keyword_set(old)) then begin
Nx=16 & Ny=16 
Nsky=17
endif else begin
Nx=16 & Ny=15
Nsky=16
endelse                          ; size lens array
exp_obj=sxpar(ho,'EXPTIME')
stime=sxpar(ho,'START')
gain=sxpar(ho,'GAIN')
ptb=double(sxpar(ho,'ROTANGLE'))
adang=double(sxpar(h,'ADANG'))
z_obj=double(sxpar(ho,'Z'))
adr=[double(sxpar(h,'ADR1')),double(sxpar(h,'ADR2'))]
eff_obj=double(sxpar(ho,'EFF_EXT'))

if image_type eq 'dobj' then begin
 ssp=readfits(wdir+'obj-sky.fts',hoo)
 eff_obj=double(sxpar(hoo,'EFF_EXT'))
endif

object_name=STRCOMPRESS(sxpar(ho,'OBJECT'))
date_obs=STRCOMPRESS(sxpar(ho,'DATE-OBS'))
;calculation mean value extintion
a=0.012
c=0.12
lambda=dindgen(N_x)*param(1)+param(0)
lamo=dindgen(N_xo)*paramo(1)+paramo(0)
secz=1./cos(z_obj*!DTOR) & print,secz
ext=(a*1./((lambda/10000.)^4)+c)*2.5/2.3
ext=0.          ;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
ext_obj=10.^(0.4*ext*secz)
adr_obj=adr*tan(z_obj*!DPI/180.)

;definition  unit
sent=sent   ;/(exp_obj*param(1))
if keyword_set(nullsent) then sent=replicate(1.,n_elements(sent))

order=fix(min(ALOG10(sent(xoffset[0]:N_x-xoffset[1]-1))))
if order lt 0 then order=order-1
out_units=10.^order		;output units in erg/cm^2/sec/A
obj_red=dblarr(N_xo,N_yo)
;cleaning cosmic hits in data cube

for k=0,N_y-1 do begin
  if min(param eq paramo) then obj_red(*,k)=spectr(*,k)*ext_obj*gain/(sent*param(1)*exp_obj*eff_obj) else $
       obj_red(*,k)=spectr(*,k)*interpol(ext_obj*gain/(sent*param(1)*exp_obj*eff_obj),lambda,lamo,/spline)
endfor
;  plot,total(spectr,2)
if not(keyword_set(nosp)) then begin
    ;determination coordinates maximum value in monochromatic image of object
  cube_obj=dblarr(N_xo,Nx,Ny)
  max_value=max(total(obj_red(N_xo/2-100:N_xo/2+100,*),1),N_max)
  yc=N_max/Nx   & xc=N_max-yc*Nx
Ny=Ny < nint(N_yo/Nx)
if ext eq '-sky' then cube_obj=reform(obj_red,N_xo,Nx,Ny) else cube_obj=reform(obj_red(*,Nsky:N_yo-1),N_xo,Nx,Ny)
 
  x=indgen(Nx)
  y=indgen(Ny)
  tmp=reform(total(cube_obj,1))
  maxx=max(total(tmp,1),xmax)
  maxy=max(total(tmp,2),ymax)

adang=37.

dl=param[1]
obj_red=reform(discor(cube_obj*1.e+17,[adr_obj[0]*dl,adr_obj[1]*dl^2],ptb-adang),$
 N_xo,Nx*Ny)*1.e-17   ;!!! correction for differential atmosphaeric refraction !!!

  cube_obj1=reform(obj_red,N_xo,Nx,Ny)

 if keyword_set(lamp) then vector_out=diaspect(cube_obj1*1.e+18,w,lambda=lamp*(z+1.),wdir=wdir,lampar=[param[0],param[1]],/nosh,cent=cent,nobgd=nobgd,old=old)*1.e-18 $
    else vector_out=diaspect(cube_obj1*1.e+18,w,/nosh,wdir=wdir,cent=cent,nobgd=nobgd,old=old)*1.e-18 
x_c=cent[0]  &  y_c=cent[1]

 obj_norm=obj_red
 for k=0,n_elements(obj_red(*,0))-1 do obj_norm(k,*)=obj_red(k,*)/max(obj_red(k,*))

max_value=fix(ALOG10(max(vector_out(50:N_xo-50)*out_units)))
max_value=10.^max_value/out_units
min_value=fix(ALOG10(min(vector_out(50:N_xo-50)*out_units)))
min_value=10.^min_value/out_units
vector_out=vector_out/max_value
ws=string(2*w+1,format='(I2)')
;vector_out=median(vector_out,3)
loadct,0
if keyword_set(plot) then begin
        window,30,xsize=700,ysize=700
        plot,lambda,vector_out,xst=1,/ylog,$
	background=255,color=0,$
	yrange=[min(vector_out(100:N_xo-100)),max(vector_out(100:N_xo-100))],psym=10,$
	ytitle='FLUX, '+string(max_value,format='(E8.1)')$
	+' erg/cm^2/cm/sec/A',$
	xtitle='wavelength, A',$
	title='total spectrum'
	xyouts,0.2,0.9,date_obs+'  '+stime+'  '+object_name+'  exposure '+$
	string(exp_obj,format='(I5)')+' sec',/norm,color=0
endif
;save result in POSTSCIPT FILE
set_plot,'PS'
        device,file=wdir+image_type+'.ps',/landscape,xsize=26,ysize=18,xoffset=1,yoffset=28
        if max(vector_out(100:N_xo-100))/min(vector_out(100:N_xo-100)) gt 10. then ify=1. else ify=0.
        plot,lamo(0:n_elements(lamo)-1),vector_out(0:n_elements(lamo)-1),ylog=0,background=255,color=0,$
	yrange=[-.05,1.]*max(vector_out(100:N_xo-100)),psym=10,$
	ytitle='FLUX, 10!U'+string(alog10(max_value),format='(I5)')$
	+'!N erg/cm!U2!Ns'+string("305B),xrange=[min(lambda),max(lambda)],$
	xtitle='wavelength,'+string("305B),yst=1,xst=1,$
	title='total spectrum, '+strtrim(string(w,format='(F5.1)'),2)+'" diaphragm'
	xyouts,0.2,0.9,date_obs+'  '+stime+'  '+object_name+'  exposure '+$
	string(exp_obj,format='(I5)')+' sec',/norm,color=0
device,/close
endif

if os_family() eq 'unix' then set_plot,'X' else set_plot,'WIN'
cont1:
  sxaddpar,ho,'XCENTER',x_c
  sxaddpar,ho,'YCENTER',y_c
  sxaddpar,ho,'DIAPHR',w
sxaddpar,ho,'CRVAL1',paramo[0]
sxaddpar,ho,'BUNIT','!Uerg!N/!Dcm!E2!N!Ds'+string("305B),befor='COMMENT'
sxaddpar,ho,'FILE',image_type+'_abs.fts',' '
writefits,wdir+image_type+out+'.fts',obj_red,ho

if not(keyword_set(nosp)) then begin
  hos=ho
;  sxaddpar,hos,'XCENTER',x_c
;  sxaddpar,hos,'YCENTER',y_c
  writefits,wdir+'spect_'+image_type+out+'.fts',vector_out*max_value,hos
endif

sxaddpar,ho,'BUNIT','relat.units',befor='COMMENT'
sxaddpar,ho,'FILE',image_type+'_rel.fts',' '
writefits,wdir+image_type+'_rel.fts',obj_norm,ho

; convertion to photon numbers::

obj_nph=dblarr(N_x, N_y)
; energy=5.e+7/lambda
; for k=0,n_elements(lambda)-1 do obj_nph(k,*)=obj_norm(k,*)/energy
obj_nph=spectr*gain

sxaddpar,h,'BUNIT','photon numbers/(sec*angstrom)',befor='COMMENT'
sxaddpar,h,'FILE',image_type+'_nph.fts',' '
writefits,wdir+image_type+'_nph.fts',obj_nph,ho

END
