pro CREA_DISPER,LOGFILE,FWHM,N_deg,YFIT=yfit,PLOT=plot,add5577=pos5577
;+
; NAME:
;	CREA_DISPER
; PURPOSE:
;      Identefication comparison spectra & creation array coefficient dispersion curve
; DESCRIPTION:
;	The routine compare position lines in  2D-array comparison MPFS-spectrum
;	(from file 'neon_s.fts') with tabulated wavelength from file 'LINES.TAB',
;	 and create after 2-dimensional
;	 polynomial approximation array dispersion coefficients
;
; CALLING SEQUENCE:
;	CREA_DISPER,LOGFILE,FWHM,N_deg,PLOT=plot
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOG = String scalar of file name  FITS-header from the LOG observation
;	FWHM = width spectral lines in px
;	N_deg = degree of 2D-polynomial approximation dispersion curve
;
; OUTPUTS:
;	 2-dimensional ( N_deg+1) x N_spectra double array coefficient dispersion curve,
;        saved to the disk in  working directory with  standard name 'disper.fts' and
;	 print in file 'disper.txt'.
;	 Last value in every string is rms approximation i px
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	PLOT - if present 2-d errors of approximation plotted to display,
;	else save plot to POSTSCRIPT file in working directory with name 'err_line.ps'
;
; RESTRICTIONS:
;	no
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   DEF_WDIR,DEF_RDIR,DEF_NAME,READ_FTS,INP_WAVE
;	Procedures:  CRSPROD,GOODPOLY
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       Removed some bags : A.Moiseev, Mar 2000
;       Continued debagging and depacking: P.Abolmasov and A.Shipulin, Jul 2003
;	Last major bag removed by P.Abolmasov, Aug 2003
;-

on_error,0
tabfile='~/standards/lines.tab'                     ; !!!

message,'creation dispersion curve',/cont
wdir=DEF_WDIR(LOGFILE)
name_neon=DEF_NAME(LOGFILE,'NEON',N_name)
dir=DEF_RDIR(LOGFILE)
out_str='comparison spectrum files: '+DEF_RDIR(LOGFILE)

for k=0,N_name-1 do begin
  out_str=out_str+name_neon(k)+','
endfor

wave=INP_WAVE(LOGFILE)                 ; wave(0) - central wavelength (lambda_c), wave(1) - estimate dispersion ()
lambda_c=wave(0)  & d_lambda=wave(1)
;definition level separation lines
if d_lambda le 1. then lev=1.
if d_lambda gt 1 and d_lambda lt 2 then lev=2.
if d_lambda gt 2 and d_lambda lt 4 then lev=3.         ; lev=3. if d_lambda eq 2.6 
if d_lambda gt 4 then lev=4.

tresh=1.

sw=0

if keyword_set(plot) then sw=1             ; sw - plotting flag

lev=lev-1						; lev=1.5
;reading neon data
neon=READ_FTS(wdir+'neon_s.fts')                       ; ,SILENT='silent')

a=size(neon)
Nx=a(1)  &  Ny=a(2)

; **********************************
if keyword_set(pos5577) then begin
   ; adding 5577 line from obj
   im=READ_FTS(wdir+'obj_os.fts')
   sky=dblarr(nx,ny)
   for y=0,ny-1 do begin
      vec=im(pos5577(0):pos5577(1),y)
      vec=vec-median(vec)
      neon(pos5577(0):pos5577(1),y)=neon(pos5577(0):pos5577(1),y)+vec
   endfor

endif
; **********************************

;read table of lines comparison spectrum
openr,1,tabfile
a=dblarr(3)
readf,1,a
table=a
lines=a[1]  &  weigth=a[2]  & intensity=a[3]
WHILE NOT EOF(1) do begin
  readf,1,a
  table=[table,a]
lines=[lines,a[1]]  &  weigth=[weigth,a[2]]  & intensity=[intensity,a[3]]
ENDWHILE

if keyword_set(pos5577) then table=[table,'5577.345 1  0.2   ']

close,1
a=size(table)
;  print,lines,':',weigth,':',intensity

lambda=dindgen(Nx)*d_lambda+lambda_c-Nx/2*d_lambda
lambda_beg=lambda(0)+10.*d_lambda
lambda_end=lambda(Nx-1)-d_lambda*10.
index=WHERE(lines gt lambda_beg and lines lt lambda_end)  ; numbers of lines inside the spectral range

lines=lines(index)
weigth=weigth(index)
intensity=intensity(index)

print,'Covered spectral range:',min(lines),'..',max(lines)

;creation full table spectrum in observed coverage
pos_previus=(lines-lambda_c)/d_lambda+Nx/2                ;   estimate line position 
s=size(pos_previus)
N_lines=s(1)
message,'Number lines in observed spectral coverage'+string(n_lines),/cont
;message,'Number of lines used'+string(N_elements(lines)),/cont
;creation tabulated comparison spectra
sigma=fwhm/2.35
x=dindgen(Nx)
neon_tab=dblarr(Nx)
                                                      ; neon_tab -- modelled neon spectrum 
;creation spectrum, sorted by LEVEL
index=WHERE(weigth gt lev)
lines=lines(index)
weigth=weigth(index)
intensity=intensity(index)
pos_previus=(lines-lambda_c)/d_lambda+Nx/2              ; linear approximation
s=size(pos_previus)
N_lines=s(1)
message,'Number lines used for creating dispersion curve'+string(n_lines),/cont
;creation tabulated comparison spectra
for k=0,N_lines-1 do begin
   gauss=(x-pos_previus(k))/sigma
   gauss=intensity(k)*exp(-gauss^2/2)
   neon_tab=neon_tab+gauss
endfor

;determination shifting central wavelength

m=50
x_shift=dblarr(Ny)

for y=0,Ny-1 do begin
  CRSPROD,neon(*,y),neon_tab,m,CORR             ; cross-correlation
  tmp=max(CORR,N_max)
  x_shift(y)=N_max-m+1                          ;  N_max - maximal element number
endfor

y=dindgen(Ny)

f=goodpoly(y,x_shift,3,3.,Y_fit)
y=dindgen(Ny)
dy=poly(y,f)                    ; dy is the polinom itself... and it's x-shift, really
pos_lines=dblarr(N_lines,Ny)

for k=0,N_lines-1 do begin
  pos_lines(k,*)=pos_previus(k)+dy(*)   ; pos_lines are positions along the dispertion (x)
endfor

w=FWHM
vector_tmp=dblarr(Nx)
posm=dblarr(N_lines)
posm0=total(pos_lines,2)/double(Ny)
spem=total(neon,2)

for k=0,N_lines-1 do begin
  vector=spem(round(posm0[k]-w):round(posm0[k]+w))
  xv=x(round(posm0[k]-w):round(posm0[k]+w))
  gf=gaussfit(xv,vector,g0,nterms=5)
  posm[k]=g0[1]
endfor

for j=0,Ny-1 do begin
  vector_tmp=neon(*,j)
  for k=0,N_lines-1 do begin
    pos=fix(pos_lines(k,j))

    ;definition local maximum
    tmp=max(vector_tmp((pos-w-1)>0:(pos+w+1)<(Nx-1)),Nmax)     
    pos=(Nmax+pos-w)<(Nx-1-w-1)                             ; pos - improved maximum coordinate
    level=(vector_tmp((pos-w)>0)+vector_tmp(pos+w))/2          ; arithmetic mean between two ends of the inteval 
    flux=total(vector_tmp(pos-w:pos+w)-level)                                 ; flux over level
 ;   pos_lines(k,j)=total((vector_tmp(pos-w:pos+w)-level)*x(pos-w:pos+w))/flux
     gf=gaussfit(x(pos-w:pos+w),vector_tmp(pos-w:pos+w),g,nterms=5,estmates=[max(vector_tmp)-min(vector_tmp),$
               posm[k],w/2.35,min(vector_tmp),0.])
    pos_lines(k,j)=g[1]
    ;level=min(vector_tmp(pos-w-1:pos+w-1))
    ;pos_lines(k,j)=find_pol(x,vector_tmp-level+0.1,pos)
  endfor
endfor

y_out=Ny/2
if sw eq 1 then window,10,xsize=Nx,ysize=Ny*2
vector=neon(*,y_out)*1000./max(neon(*,y_out))
if sw eq 1 then plot,x,vector,xst=1,yst=1,yrange=[-10,1300]

for k=0,N_lines-1 do begin
  pos=pos_lines(k,y_out)
  if sw eq 1 then oplot,[pos,pos],[vector(pos)+60,vector(pos)+80]
  if sw eq 1 then xyouts,pos,vector(pos)+90,string(lines(k),format='(F7.2)'),/data,orientation=90
endfor

rms=dblarr(Ny) & rmsg=dblarr(Ny)
;*********************************************************

pos_lines_fit=pos_lines
if keyword_set(yfit) then begin
  ;polynomial fitting position line along pseudo-slit
  y_pos=dindgen(Ny)
  N_lin=N_elements(lines)

  nn_x=16                  ; MPFS field
  nn_y=16                  ; empty space
  y_pos1=dindgen(nn_x)
  for j=0,nn_y-1 do begin             ; for every line (k) and for every row in the datacube
   for k=0,N_lin-1 do begin
    f=goodpoly(y_pos1,double(pos_lines(k,j*nn_x:(j+1)*nn_x-1)),N_deg,2.,Yfit)
    pos_lines_fit(k,j*nn_x:(j+1)*nn_x-1)=Yfit
   endfor
  endfor

;  goto,laja 
 
  for k=0,N_lin-1 do begin
    ;f=goodpoly(y_pos,double(pos_lines(k,*)),2,2,Yfit)
    f=goodpoly(y_pos,double(pos_lines_fit(k,*)),N_deg,1.,Yfit)
    if k eq 0 then begin
      plot,y_pos,pos_lines(k,*)-yfit,psym=2,xst=1,yst=1
      oplot,y_pos,yfit*0
    endif
    pos_lines_fit(k,*)=Yfit
  endfor
;  laja:
endif

;****************************************************
;creation array coefficients dispersion curves
disper_par=dblarr(N_deg+2,Ny)

lambda_fit=dblarr(Ny,Nx)

for y1=0,Ny-1 do begin
  f=goodpoly(double(pos_lines_fit(*,y1)),double(lines),N_deg,tresh,Yfit,xpk,ypk)
  disper_par(0:N_deg,y1)=f(*)
  ;if y1 eq 120 then begin
  ; plot,pos_lines_fit(*,y1),lines-yfit,psym=2,xst=1,yst=1
  ; oplot,pos_lines_fit(*,y1),yfit*0
  ;endif

  lines_fit=0
  ;lambda_fit=0
  
  for j=0,N_deg do begin
    lines_fit=lines_fit+f(j)*pos_lines(*,y1)^j
  endfor

  lambda_fit(y1,*)=interpol(double(pos_lines_fit(*,y1)),double(lines),double(lambda),/spline)
  rms(y1)=stdev(lines_fit-lines)
  rmsg(y1)=stdev(ypk-poly(xpk,f))
endfor

;lambda_fit=interpol(pos_lines_fit,lines,lambda,/spline)

plot,lambda,lambda_fit(Ny/2,*)
oplot,lambda,(lambda-lambda_c)/d_lambda+Nx/2 

; plot,lambda,neon(dindgen(Nx)+x_shift(Ny/2),Ny/2),title='Multigauss fitting neon spect' 
; oplot,lambda,neon_tab(dindgen(Nx))*max(neon)/max(neon_tab),color=89078                        ;,color=10000

;  rms=rms/d_lambda & rmsg=rmsg/d_lambda
disper_par(N_deg+1,*)=rms(*)/d_lambda
rms_mean=total(rms)/Ny & rmsg_mean=total(rmsg)/Ny
x_hist=dindgen(21)*0.025
if sw eq 1 then window,12,xsize=400,ysize=400
if sw eq 1 then plot,x_hist,histogram(rms,min=0,max=.5,binsize=.025),$
	xst=1,yst=1,psym=10,title='distribution errors approximation',$
	xtitle='error, px'
;*********************************************************************
;calculation and plot errors approximation
N_lin=N_elements(lines)
lines_calc=dblarr(N_lin,Ny)
for y=0,Ny-1 do begin
  for j=0,N_deg do begin
    lines_calc(*,y)=lines_calc(*,y)+disper_par(j,y)*pos_lines(*,y)^j
  endfor
endfor

dy=2.0
if sw eq 0 then set_plot,'PS'
if sw eq 0 then device,file=wdir+'err_line.ps',/portrait,xsize=18,ysize=26,yoffset=0
if sw eq 1 then Window,13,xsize=700,ysize=750
if sw eq 1 then loadct,27
plot,[0,Ny],[-1,N_lin]*dy,xst=1,yst=1,/nodata,charsize=0.75,$
	position=[0.05,0.05,0.8,0.95],/norm,$
	title='errors approximation dispersion curve mean r.m.s='+$
	string(rms_mean,format='(F7.3)')+'('+string(rmsg_mean,format='(F7.3)')+') A',$
	xtitle='number of spectra',ytitle='!4Dk, !3px'
xyouts,0.1,0.005,out_str,/norm
xyouts,0.1,0.98,'number lines '+string(N_lines,format='(I2)')+$
	'  central wavelength '+string(lambda_c,format='(I4)')+$
	' A  dispersion '+string(d_lambda,format='(F5.2)')+$
	' A/px'+' Ndeg='+string(N_deg,format='(I2)'),/norm

for k=0,N_lin-1 do begin
  rms=stdev(-lines_calc(k,*)+lines(k),mean)
  oplot,dindgen(Ny),(-lines_calc(k,*)+lines(k)-mean)/d_lambda+dy*k,psym=6,symsize=0.2,color=0
  oplot,[0,Ny],[dy,dy]*k,color=0
  xyouts,Ny*1.02,dy*k,string(lines(k),format='(F7.2)')+$
		string(mean,format='(F8.2)')+' !9+!3'+$
		string(rms,format='(F5.2)'),charsize=0.8,/data
endfor
 xyouts,Ny*1.03,dy*(N_lin-0.5),'!4k, !3A',charsize=0.75,/data
 xyouts,Ny*1.12,dy*(N_lin-0.5),'!4Dk, !3A',charsize=0.75,/data
if sw eq 0 then device,/close
if sw eq 0 then begin
  if os_family() eq 'unix' then set_plot,'X' else set_plot,'WIN'
endif

;save dispersion curve

writefits,wdir+'disper.fts',double(disper_par)
a=size(disper_par)
out_format='(F8.2,F8.5,'+string(a(1)-3,format='(I1)')+'E16.6,F6.2)'
Openw, UNIT,wdir+'disper.txt', /GET_LUN

for k=0,a(2)-1 do begin
  printf, UNIT,disper_par(*,k),format=out_format
endfor

close, UNIT
message,'mean error dispersion curves ='+string(rms_mean,format='(F5.2)')+' px',/cont
disper_par=0
cont:
END
