pro DEF_SENT,LOGFILE,PLOT=plot,MANUAL=manual,ext=ext,not_fon=not_fon,xoffset=xoffset
;+
; NAME:
;       DEF_SENT
; PURPOSE:
;      definition  spectral sensitivity
; DESCRIPTION:
;       routine used linearised MPFS-spectra of standard star (filename in working
;       directory 'star_sky.fts'), information from FITS-header about total exposure,
;       gain CCD, mean value zenit distance for calculation observed flux. Observed
;       flux compare with absolute energy distribution star.
;       Routine used ASCII data files for the Oke (AJ,99, 1621, 1990) optical
;       spectrophotometric standard stars
; CALLING SEQUENCE:
;       DEF_SENT, LOGFILE, PLOT=plot
; CATEGORY:
;       reduction MPFS-data
; INPUTS:
;       LOGFILE = file name of LOG observation (in format FITS-header)
; OUTPUTS:
;        in working directory saved 1-dimensional  double array spectral sensitivity
;        in  erg/cm^2/sec/A per ADU (standard filename 'sent.fts');
; OPTIONAL OUTPUT:
;       no
; OPTIONAL INPUT KEYWORDS:
;       PLOT - if present the  curve DQE  plotted to display,
;       else save plot to POSTSCRIPT file in working directory with name 'DQE.ps'
;       xoffset - lower and upper limits for spectrum fitting and extraction
;
; RESTRICTIONS:
;       no
; NOTES:
;       no;
; PROCEDURES USED:
;       Function :   DEF_WDIR,NAME_TAB,READ_FTS,INP_WAVE,OS_FAMILY
;       Procedures:  SXPAR,SXADDPAR,MKHDR
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;-      Corrected and primarily reducted by students of Stellar Physics Lab., SAO RAS, Jul 2003 


on_error,0
!P.background=255 & !P.color=0
if keyword_set(plot) then sw=1 else sw=0                            ; sw -- plotting flag
wdir=DEF_WDIR(LOGFILE)

if not(keyword_set(ext)) then ext='-sky'
star_lin=READ_FTS(wdir+'star'+ext+'.fts',h_star,/sil) ;
a=size(star_lin)
; if keyword_set(xoffset) then star_lin=star_lin(xoffset[0]:a[1]-xoffset[1]-1,*) else xoffset=[0,0]
a=size(star_lin)

eff=sxpar(h_star,'EFF_EXT')
;eff=0.94
param=[sxpar(h_star,'CRVAL1'),sxpar(h_star,'CDELT1'),sxpar(h_star,'NAXIS1')]
; param[2]=a[1]
; param[0]=param[0]+param[1]*xoffset[0]
ADU=sxpar(h_star,'GAIN')
table_name=NAME_TAB(LOGFILE)
message,'reading standard star fluxes from file: '+table_name,/cont

table=read_asc(table_name)
; table=reform(table,2,n_elements(table)/2)

lambda_tab=reform(table(0,*))
f_tab=reform(table(1,*))*1.D-16           ; table flux (former mag_tab)

T_exp=sxpar(h_star,'EXPTIME')                                        ; exposure time
z_star=double(sxpar(h_star,'Z'))                                      ; zenith distance
ptb=double(sxpar(h_star,'ROTANGLE'))                                      ; zenith distance
object_name=STRCOMPRESS(sxpar(h_star,'OBJECT'))                      ; 
date_obs=STRCOMPRESS(sxpar(h_star,'DATE-OBS'))                       ; observation date
Nz=param(2)                                                          ; cube size along dispertion
;creation total spectrum sky substracted star in ADU/sec/A

Nx=16  & Ny=16

;transformation in data cube format
Ny=floor(a[2]/Nx)
if Ny ne 16 then message,'16X'+strtrim(string(Ny,for='(I10)'),2)+' frame!',/cont
cube_star=dblarr(Nz,Nx,Ny)
cube_star=star_lin(0:Nz-1,0:Nx*Ny-1)
N_x=Nx  &  N_y=Ny
map=dblarr(N_x,N_y)
vector=dblarr(Nz)
; for i=0,Nx-1 do for j=0,Ny-1 do cube_star(*,i,j)=med_ext(cube_star(*,i,j),3)
 adr=1.
; for k=0,Nz-1 do vector[k]=total(cube_star(k,*,*)-bormed(reform(cube_star(k,*,*))))
adang=1.
vector=diaspect(cube_star,3.,lampar=[param[0],param[1]],adr=adr,adang=adang,wdir=wdir,ptb=ptb)       ; ADU from star on k 
adang=-adang*180./!DPI+ptb                                        ; Afanasiev's constant, deg

dl=param[1]
adr=[adr[0]/dl,adr[1]/dl^2]/(tan(z_star*!DPI/180.))  ;  estimated atmosphaeric dispersion rate (in arcsec/A, linear approximation for plane-parallel atmosphere)

;creation observed magnitude standard star
lambda=dindgen(param[2])*param[1]+param[0]
badlam=[4340.,4861.,5577.345,6300.,6563.,6880.]
; index=where(lambda_tab gt param(0) and $
;               lambda_tab lt param(0)+param(1)*param(2))

; lambda_tab=lambda_tab(index)
; f_tab=f_tab(index)
; lambda=dindgen(param(2))*param(1)+param(0)
f_obs=INTERPOL(f_tab,lambda_tab,lambda)*T_exp*param[1]*eff                ; table interpolated data! former mag_obs

a=0.012 & c=0.12
extin=(a*1./((lambda/10000.)^4.)+c)*1.086
;;extin=0.          ;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
f_obs=f_obs*10^(-.4*extin/cos(z_star*!DTOR))    ;+3.15

; print,'extinction factor = '
; print,10^(-.4*extin(*)/cos(z_star*!DTOR))

; Flux_obs=3.39E-9*10^(-0.4*mag_obs) ; Jy
; Flux_obs=10^(-0.4*(mag_obs+48.59)) ; Jy

;calculation quantum efficiency & spectral sensitivity
N_obs=f_obs
; N_obs=10^(-0.4*mag_obs)*948.0*S/ADU
; N_obs=N_obs*(5500.0/lambda)^2*9.9105E-17    ; in mJy - all depends on the value given by Oke -- damn mJy! 

cut=0.05 & ymax=0.05
N=N_elements(vector)
if max((vector/N_obs)) gt 0.05 then ymax=0.1
if max((vector/N_obs)) gt 0.1 then ymax=0.2
w=15.  &  nlag=20

vector1=lowess(lambda,vector,w*10.,1)
N_obs1=lowess(lambda,N_obs,w*10.,1)

lag=indgen(nlag)*2.-nlag
corr=smooth(c_correlate(vector/vector1,N_obs/N_obs1,lag),7)
maxv=max(corr,wmax)
 DQE=shift(vector,lag[wmax])/(N_obs)                   ; (spectral sensivity)

;calculation average sensitivity

x_pos=indgen(Nz)  &  y_pos=smooth(med_ext(vector/N_obs,w),w)

dw=w*param[1]

lambda1=lambda
w1=indgen(Nz)
;for q=0,n_elements(badlam)-1 do w1=w1(where(abs(lambda(w1)-badlam[q]) gt 1.6*w))
;dqe_int=interpol(dqe(w1),lambda(w1),lambda)
dqe_int=dqe
DQE_average=LOWESS(lambda,med_ext(dqe_int,w),dw*10.,2)

if keyword_set(manual) then begin
  DQE_average=MANUAL_FIT(DQE,lambda,SPLINE_SET='spline_set')
endif
; DQE_average=smooth(med_ext(med_ext(DQE_int,3.*w),3.*w),2.*w)

if sw eq 1 then Window,20
if sw eq 0 then begin
  SET_PLOT,'PS'
  device,file=wdir+'DQE.ps',/portrait,xsize=16,ysize=22,yoffset=0,xoffset=0,/color
!p.multi=[0,1,2]
endif
eff=1.
loadct,0,/sil
plot,lambda,DQE_average/EFF,xst=1,yst=1,yrange=[0.,max(DQE_average(xoffset[0]:Nz-1-xoffset[1])/EFF)*1.5],$
        title='DQE '+date_obs+' star '+object_name+$
        ' exposure '+string(T_exp,format='(I4)')+' sec',$
        xtitle='wavelength, A',ytitle='total quantum efficincy, %'
loadct,27,/sil
oplot,lambda,DQE/eff,color=1000
; oplot,lambda(x_pos),y_pos/eff,psym=6
loadct,0,/sil
plot,lambda,f_obs,/xst
loadct,27,/sil
oplot,lambda,vector*median(f_obs/vector),color=1000
loadct,0,/sil
if sw eq 0 then begin
!p.multi=0
  device,/close
  if os_family() eq 'unix' then set_plot,'X' else set_plot,'WIN'
endif

;sent=3.39E-9*ADU/(9.48*DQE_average*S)
sent=DQE_average(xoffset[0]:n_elements(DQE_average)-1-xoffset[1])*ADU
mkhdr,h_sent,sent
sxaddpar,h_sent,'CRVAL1', param(0)+param[1]*xoffset[0]
sxaddpar,h_sent,'CDELT1', param[1]
sxaddpar,h_sent,'ADR1',adr[0],'atmosphaeric dispersion in optical range, arcsec/Angstrom'
sxaddpar,h_sent,'ADR2',adr[1],'atmosphaeric dispersion, second order, arcsec/Angstrom^2'
; sxaddpar,h_sent,'ADR3',adr[2],'atmosphaeric dispersion, third order, arcsec/Angstrom^3'
sxaddpar,h_sent,'BUNIT', 'erg/cm^2/sec/A'
sxaddpar,h_sent,'ADANG',adang,'table null rotation angle'
writefits,wdir+'sent.fts',sent,h_sent
END
