pro EXTRACT,LOGFILE,fwhm,image_type,add_clean=add_clean,ext=ext,ysh=ysh
;+
; NAME:
;	EXTRACT
; PURPOSE:
;	extraction spectra in frame MPFS-image
; DESCRIPTION:
;	Routine extracted spectra in MFPS-frame in file with standard name '*_i.fts'
;	('*'=image_type) along traectory from file with standard name 'fib_tra.fts',
;	for obj-fibers, and along traectory etalon (from file 'eta_tra.fts' for sky
;	Routine modified FITS-header
;	Three kinds of extracted datacubes produced using object, sky and all fibers ('_s', '_ns' and '_os', respectively)
; CALLING SEQUENCE:
;	EXTRACT, LOGFILE, fwhm, image_type
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOGFILE = file name of LOG observation (in format FITS-header)
;	fwhm = width band of spectra
;	image_type = string scalar type exposure (values: 'obj','star','eta',
;						'neon','test')
;	flat-field exctartion with traectories from flat_tra.fts
;
; OUTPUTS:
;	Result - array float point spectra saved to disk in working directory
;	with standard name '*_s.fts'.
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	add_clean - flag for making additional median clean in y-direction (appropriate for spectra with too much cosmic hits)
;
; RESTRICTIONS:
;	no
;
; NOTES:
;	no
;
; PROCEDURES USED:
;	DEF_WDIR,SXADDPAR
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       Read and revised by Pavel Abolmasov, Special Astrophysical Observatory RAS, Jul 2003
;	Flat field extraction added by P. Abolmasov and A. Shipulin, Jan 2004
;

message,'extraction spectra in '+image_type+'-image',/cont
wdir=DEF_WDIR(LOGFILE)

if not(keyword_set(ext)) then ext='_i'

image=READ_FTS(wdir+image_type+ext+'.fts',head_image,SILENT='silent')
if image_type eq 'star' or image_type eq 'test' then begin
 traectory_full=READ_FTS(wdir+'flat_tra_star.fts',SILENT='silent')
 traectory_obj=READ_FTS(wdir+'fib_tra_star.fts',SILENT='silent')
 traectory_sky=READ_FTS(wdir+'sky_tra_star.fts',SILENT='silent')
 sig_full=READ_FTS(wdir+'flat_sig_star.fts',SILENT='silent')
 sig_obj=READ_FTS(wdir+'fib_sig_star.fts',SILENT='silent')
 sig_sky=READ_FTS(wdir+'sky_sig_star.fts',SILENT='silent')
endif else begin
 traectory_full=READ_FTS(wdir+'flat_tra.fts',SILENT='silent')
 traectory_obj=READ_FTS(wdir+'fib_tra.fts',SILENT='silent')
 traectory_sky=READ_FTS(wdir+'sky_tra.fts',SILENT='silent')
 sig_full=READ_FTS(wdir+'flat_sig.fts',SILENT='silent')
 sig_obj=READ_FTS(wdir+'fib_sig.fts',SILENT='silent')
 sig_sky=READ_FTS(wdir+'sky_sig.fts',SILENT='silent')
endelse

a=size(image)
Nx=a(1)  & Ny=a(2)
b=size(traectory_full)
Nflat=b(2)
b=size(traectory_obj)
Nobj=b(2)
b=size(traectory_sky)
Nsky=b(2)
flat=dblarr(Nx,Nflat)
obj=dblarr(Nx,Nobj)
sky=dblarr(Nx,Nsky)
xar=dblarr(Nx,Ny)  &  yar=dblarr(Nx,Ny)
xar(*)=dindgen(Nx*Ny) mod Nx
yar(*)=dindgen(Nx*Ny)/double(Nx)

if keyword_set(ysh) then begin
  at=size(traectory_full)
  ysht=total(traectory_full,2)/double(at[2])  ; all-fibers' average
  if image_type eq 'star' or image_type eq 'test' then eta=READ_FTS(wdir+'eta_star_i.fts',head_eta,SILENT='silent')$
 else eta=READ_FTS(wdir+'eta_i.fts',head_eta,SILENT='silent')
  ae=size(eta)
  
  image1=image*0.
for k=0,Nsky-1 do begin
  image1(where((yar-traectory_sky((xar),k))^2 le 50.))=image(where((yar-traectory_sky((xar),k))^2 le 50.))
endfor
  eta_int=interpol(total(eta(800:1200<a[1]-1,*),1),dindgen(Ny),dindgen(Ny*10.)/10.,/spline)  
  ima_int=interpol(total(image1(800:1200<a[1]-1,*),1),dindgen(Ny),dindgen(Ny*10.)/10.,/spline)  

  crsprod,eta_int,ima_int,40,cross
  maxv=max(cross,wmax)
  yshift=-double(wmax-40)/10.

  traectory_full=traectory_full+yshift
  traectory_obj=traectory_obj+yshift
  traectory_sky=traectory_sky+yshift
  message,'trajectory shifting '+strtrim(string(yshift,format='(F10.5)'),2)+'pxls',/cont
endif

;  tabulation shifting GAUSS-profile in interval [-w,w]

w=(fwhm)                                                               ; w = 4
sigma=fwhm/sqrt(8.*ALOG(2.))

a=size(image)
Nx=a(1)  & Ny=a(2)
b=size(traectory_full)
Nflat=b(2)
b=size(traectory_obj)
Nobj=b(2)
b=size(traectory_sky)
Nsky=b(2)
flat=dblarr(Nx,Nflat)
obj=dblarr(Nx,Nobj)
sky=dblarr(Nx,Nsky)

if image_type eq 'obj' or image_type eq 'star' or image_type eq 'flat' then $
	message,'extraction spectra nigth sky probe in '+image_type+'-image',/cont

sf=5          ; stretch factor
ys=dindgen(Ny*sf)/double(sf)

for x=0,Nx-1 do begin
        vector_show=interpol(total(image(x-sf>0:x+sf<Nx-1,*),1)/((x+sf<Nx-1)-(x-sf>0)+1.),dindgen(Ny),ys)
        vector=interpol(reform(image(x,*)),dindgen(Ny),ys)
       ; vector=reform(image(x,*))
     if x mod 100 eq 0 then print,'column # ',x

       yc_flat=traectory_full(x,*)
       yc_obj=traectory_obj(x,*)
       yc_sky=traectory_sky(x,*)
        for y=0,Nflat-1 do begin
          ;  yc=traectory_full(x,y)
            wy=where(abs(ys-yc_flat[y]) le w)
            yy=ys(wy)
            vv=vector(wy)
            g=exp(-((yy-yc_flat[y])/sigma)^2/2.)
            flat(x,y)=total(vv*g)/double(sf)    ; not always a flat field
            ; print,y

            if y le Nobj-1 then begin
              ; yc=traectory_obj(x,y)
             wy=where(abs(ys-yc_obj[y]) le w)
              yy=ys(wy)
              g=exp(-((yy-yc_obj[y])/sigma)^2/2.)
              vv=vector(wy)
              obj(x,y)=total(vv*g)/double(sf)    ; not always a flat field
            endif

            if y le Nsky-1 then begin
         ;     yc=traectory_sky(x,y)
              wy=where(abs(ys-yc_sky[y]) le w)
              yy=ys(wy)
              vv=vector(wy)
              g=exp(-((yy-yc_sky[y])/sigma)^2/2)
              sky(x,y)=total(vv*g)/double(sf)    ; not always a flat field
            endif
        endfor
  
     if  x eq 1000 or x eq 100 or x eq 1900  or x eq 1783 then begin
              gsim=vector*0.
              gsim_sky=vector*0.
              for k=0,Nflat-1 do begin
                gsim=gsim+exp(-((ys-traectory_full[x,k])/sig_full[x,k])^2/2.)*flat[x,k]/sqrt(!DPI*2.*sig_full[x,k]^2)
              endfor
              for k=0,Nsky-1 do begin
                gsim_sky=gsim_sky+exp(-((ys-traectory_sky[x,k])/sig_sky[x,k])^2/2.)*sky[x,k]/sqrt(!DPI*2.*sig_sky[x,k]^2)
              endfor
          set_plot,'ps'
           device,filename=wdir+'extract_'+image_type+strtrim(string(x,format='(I10)'),2)+'.ps',/color
            loadct,0
             !p.multi=[0,1,4]
              plot,ys,vector,xrange=[300,500],xst=1,yst=1
             loadct,27
              oplot,ys,gsim,color=1000
              oplot,ys,gsim_sky,color=10500
             loadct,0
              plot,ys,vector,xrange=[800,1000],xst=1,yst=1
             loadct,27
              oplot,ys,gsim,color=1000
              oplot,ys,gsim_sky,color=10100
             loadct,0
              plot,ys,vector,xrange=[1300,1500],xst=1,yst=1
             loadct,27
              oplot,ys,gsim,color=1000
              oplot,ys,gsim_sky,color=10100
             loadct,0
              plot,ys,vector,xrange=[1800,2000],xst=1,yst=1
             loadct,27
              oplot,ys,gsim,color=1000
              oplot,ys,gsim_sky,color=10100
             loadct,0
             !p.multi=0
           device,/close
          set_plot,'x'
       endif
endfor

sxaddhist,'width band of extraction='+string(FWHM,format='(I2)')+' px',head_image

if image_type eq 'star' or image_type eq 'obj' then begin
  ;y_min=traectory_full(Nx/2,0) & y_max=traectory_full(Nx/2,Nfib-1)
  eff=total(flat)/total(image)
  print,'efficiency extraction=',eff
  sxaddpar,head_image,'EFF_EXT',eff,' EFFICIENCY EXTRACTION'
endif

sxaddpar,head_image,'FILE',image_type+'_s.fts'
writefits,wdir+image_type+'_s.fts',flat,head_image

;if image_type eq 'obj' or image_type eq 'star' or image_type eq 'flat' then begin    
; ns -- nigthspectra for future subtraction

  sxaddpar,head_image,'FILE',image_type+'_ns.fts'
  writefits,wdir+image_type+'_ns.fts',sky,head_image
  sxaddpar,head_image,'FILE',image_type+'_os.fts'
  writefits,wdir+image_type+'_os.fts',obj,head_image

;endif
  
;endif
END
