pro FIND_ETA,LOGFILE,GAUSS=gauss,PLOT=plot,N_sig=N_sig,xstep=xstep,star=star,N_eta=N_eta
;+
; NAME:
;	FIND_ETA
; PURPOSE:
;      Find and creation traectory spectra etalon fibers
; DESCRIPTION:
;	Routine find etalon spectra in MPFS-frame from file 'eta_i.fts' and
;	create after polynomial approximation traectory etalon fibers
;       
; CALLING SEQUENCE:
;	FIND_ETA, LOGFILE, GAUSS=gauss, PLOT=plot,N_sig=N_sig
;
; CATEGORY:
;	reduction MPFS-data
;
; INPUTS:
;	LOG = String scalar of file name  FITS-header from the LOG observation
;       N_sig = threshold for determination ( DEFAULT = 1)
; OUTPUTS:
;	 Result saved to disk in working directory in files with standard names
;	 'eta_tra.fts' - 2D  double array traectory fibers
;	 'eta_poly.txt' - table coefficient polynomyal approximation traectory
;
; OPTIONAL OUTPUT:
;	no
;
; OPTIONAL INPUT KEYWORDS:
;	GAUSS - if present  cross-dispersion profile fitted by gauss-function
;	PLOT - if present 2-d errors of approximation plotted to display,
;	else save plot to POSTSCRIPT file in working directory with name 'err_line.ps'
;   N_sig - value for peak determination ( DEFAULT=1)
; RESTRICTIONS:
;	no
; NOTES:
;	no
;
; PROCEDURES USED:
;	Function :   DEF_WDIR,READ_FTS
;	Procedures:  FI_PEAK
;
; MODIFICATION HISTORY:
;       Written by Victor Afanasiev, Special Astrophysical Observatory RAS, Jul 1999
;       N_sig added by A.Moiseev
;       Revised by P. Abolmasov and A. Shipulin
;


!P.BACKGROUND=255 & !P.COLOR=0
message,'creation traectory etalon spectra',/cont

if not(keyword_set(xstep)) then xstep=100
 if not(keyword_set(N_sig)) then N_sig=1
wdir=DEF_WDIR(LOGFILE)
if not(keyword_set(star)) then image=READ_FTS(wdir+'eta_i.fts',SILENT='silent') else $
  image=READ_FTS(wdir+'eta_star_i.fts',SILENT='silent')

sw=0
if keyword_set(plot) then sw=1             ;sw - plot flag

sw_g=0
if keyword_set(gauss) then sw_g=1          ; sw_g - Gauss flag

if keyword_set(old) then wc=10 else   wc=50                         ; wc - estimate ~1/3 distance between two maxima in y
a=size(image)
Nx=a(1)   &   Ny=a(2)
yr=[wc,Nx-wc]
if not(keyword_set(N_eta)) then N_eta=17
N_init=N_eta
rms=dblarr(N_eta)                   ; 16 root mean squares
traectory=dblarr(Nx,N_eta)
w=xstep                                  ; w - x-step
N_col=Nx/w
x_col=(indgen(N_col)+1)*w                 ; x-coordinates colums
colum=dblarr(Ny)
eta_pos=dblarr(N_col,N_eta)              ; approximation points coordinates
eta_sig=dblarr(N_col,N_eta)              ; approximation points coordinates
y=dindgen(Ny)
good=-1           ; good - array valid x_col numbers

for k=0,N_col-1 do begin
  if a(0) eq 3 then colum=total(median(image(x_col(k)-w/2>0:x_col(k)+w/2< Nx-1,*),3),1)/(w+1) else colum=total(image(x_col(k)-w/2>0:x_col(k)+w/2< Nx-1,*),1)/(w+1)
  robomean,colum,2,0.2,mean_1,avgdev,sigma
  level=N_sig*sigma                    ; criminal level
  xpl=dindgen(Ny)
;  xpl=xpl(where(xpl le yr[1] and xpl ge yr[0]))
  bgd=lowess(xpl,colum,100,1)                  ;poly(xpl,goodpoly(xpl,colum,2,1.,yfit,newx,newy))
  fi_peak,xpl(yr[0]:yr[1]),colum(yr[0]:yr[1])-bgd(yr[0]:yr[1]),level,y_eta,xpk,ypk,bkpk,N_eta   ; x is y, y is intensity

  ;determination position etalons spectra in eath colums with N_eta=N_init
  ;y_eta - y-coordinates peaks, xpk,ypk - maxima coordinates
  ;N_eta - maximal number counter
y_eta=double(xpk)
    syrt=sort(ypk)

  if N_eta gt N_init then begin
    syrt=sort(ypk)
    syrt=syrt(N_eta-N_init:N_eta-1)
    syrt=syrt(sort(syrt))
    y_eta=y_eta(syrt) & N_eta=N_init
    ypk=ypk(syrt)
  endif

;plot,xpl,colum-bgd-level,background=0,color=-1
;oplot,xpk(syrt),ypk(syrt)-bgd-level,psym=2,color=1000 

 if (N_eta-N_init) eq 0 then begin
   good=[good,k]
   for j=0,N_init-1 do begin         ; eta_pos - y-coordinate maximum
    if sw_g eq 0 then norm=total(colum(y_eta(j)-wc/2>0:y_eta(j)+wc/2<2071))
    if sw_g eq 0 then eta_pos(k,j)=total((colum(y_eta(j)-wc/2>0:y_eta(j)+wc/2<2071))*y(y_eta(j)-wc/2>0:y_eta(j)+wc/2<2071))/norm
    if sw_g eq 1 then begin
      yg=y((y_eta(j)-wc/2)>0:(y_eta(j)+wc/2)<Nx-1)
      veg=colum((y_eta(j)-wc/2)>0:(y_eta(j)+wc/2)<Nx-1)
      fg=gaussfit(yg,veg,G,nterms=4)
   ;   rms=sqrt((fg-veg)^2/n_elements(yg))
;      wr=where(fg le max(fg)-.1*(max(fg)-min(fg)))
;      fg=gaussfit(yg(wr),veg(wr),G,nterms=4)
      eta_pos[k,j]=G[1]
      eta_sig[k,j]=G[2]
;      plot,yg,veg
;      oplot,yg,fg,color=10000
      fg=fg
    endif
   endfor
  endif
  if keyword_set(plot) then print,'column ',$
	string(k,format='(I2)'),' number peak ',string(N_eta,format='(I2)')           ;,string(mean_1),string(avgdev)
endfor

N_col=N_elements(good)-1                    ; N_col - good colums number
eta_pos_good=dblarr(N_col,N_init)
; N_eta=17

for k=0,N_eta-1 do begin
  c=where(eta_pos(*,k) le 0 or eta_pos(*,k) ge Ny,indec)
  b=where(eta_pos(*,k) gt 0 and eta_pos(*,k) lt Ny,indeb)
  eta_pos_good(b,k)=eta_pos(b,k)
  if indec gt 0 then eta_pos_good(c,k)=interpol(eta_pos(b,k),double(b),double(c))
endfor

x_col=x_col(b)
eta_pos=eta_pos_good
loadct,0

set_plot,'PS'
  if not(keyword_set(star)) then device,file=wdir+'geometry.ps',/portrait,xsize=18,ysize=26,yoffset=0 else $
        device,file=wdir+'geometry_star.ps',/portrait,xsize=18,ysize=26,yoffset=0
plot,[0,Nx],[-5.,5.],xst=1,yst=1,/nodata,$
	xtitle='x, px',ytitle='!4D!3y, px',$
	title='accuracy geometrical model etalon'
x=dindgen(Nx)
close,1
if not(keyword_set(star)) then openw,1,wdir+'eta_poly.txt' else openw,1,wdir+'eta_star_poly.txt'

tra_width=traectory

for i=0,N_eta-1 do begin
;  oplot,x_col,eta_pos(*,i)-median(eta_pos(*,i)),psym=1,symsize=0.5
 ; oplot,x_col,eta_pos(*,i)-median(eta_pos(*,i)),eta_sig(*,i)
;  l2000=where(eta_pos(*,i) lt 2000.)
  b=goodpoly(double(x_col),double(eta_pos(*,i)), 2, 3.,f,newx,newy)
  b=goodpoly(newx,newy,6,2.,yfit,newx,newy)
  f=poly(x,b)
;  f=smooth(med_ext(eta_pos(*,i),10),10)
  traectory(*,i)=f  ;interpol(f,x_col,x,/spline)

if sw_g eq 1.  then b=goodpoly(double(x_col),double(eta_sig(*,i)), 2, 3.,f,newx,newy)
b=goodpoly(newx,newy,6,2.,yfit,newx,newy)
  f=poly(x,b)
;  f=smooth(med_ext(eta_pos(*,i),10),10)
  tra_width(*,i)=f  ;interpol(f,x_col,x,/spline)
 
  oplot,x_col,replicate(double(i-N_eta/2.)*.5,n_elements(x_col)),linestyle=3,color=1
  oplot,x_col,(eta_pos(*,i)-traectory(x_col,i))*1.+double(i-N_eta/2.)*.5,color=1
;  oploterr,x_col,eta_pos(*,i)-traectory(x_col,i)+double(i-N_eta/2.)*.5,tra_width(x_col,i)/20.,3
  xyouts,x_col[0],double(i-N_eta/2.)*.5,string(stdev(eta_pos(*,i)-f(x_col)),format='(F7.5)')
  printf,1,stdev(eta_pos(*,i)-f(x_col)),format='(F8.2,4E14.5,F5.2)'
  rms(i)=stdev(eta_pos(*,i)-f(x_col))
endfor

err_model=string(total(rms)/17.,format='(F4.2)')

MESSAGE,'accuracy geometrical model etalon frame '+err_model+' px',/cont
device,/close

  if os_family() eq 'unix' then set_plot,'X' else set_plot,'WIN'

close,1

if not(keyword_set(star)) then begin 
 writefits,wdir+'eta_tra.fts',traectory
 writefits,wdir+'eta_sig.fts',tra_width
 set_plot,'ps'
 device,filename=wdir+'geometry_louse.ps',/land,/color
   loadct,3
   tvscl,-(smooth(image,15)),1.,1.,xsize=15,ysize=15,/cent
   loadct,0
   for k=0,N_eta-1 do plot,traectory(*,k),xrange=[0,Nx],yrange=[0,Ny],pos=[1.,1.,16.,16.]*1000.,/dev,$
      /noer,color=1,xst=1,yst=1,thick=2.,linestyle=2
 device,/close
 set_plot,'x'
endif else begin
 writefits,wdir+'eta_star_tra.fts',traectory
 writefits,wdir+'eta_star_sig.fts',tra_width
 set_plot,'ps'
 device,filename=wdir+'geometry_louse_star.ps',/land,/color
   loadct,3
   tvscl,-(smooth(image,15)),1.,1.,xsize=15,ysize=15,/cent
   loadct,0
   for k=0,N_eta-1 do plot,traectory(*,k),xrange=[0,Nx],yrange=[0,Ny],pos=[1.,1.,16.,16.]*1000.,/dev,$
      /noer,color=1,xst=1,yst=1,thick=2.,linestyle=2
 device,/close
 set_plot,'x'

endelse
 eta_pos=0
 traectory=0
END
