PRO medarr, inarr, outarr
;+
; NAME:
;	MEDARR
; PURPOSE:
;	To combine a series of arrays into a single array by filling each
;	pixel in the output array with the median of the corresponding pixels
;	in the input arrays.   Useful, for example to combine a stack of CCD
;       images, while removing cosmic ray hits.
; CALLING SEQUENCE:
;	MEDARR, inarr, outarr
; INPUTS:
;	inarr  -- A three dimensional array containing the input arrays to 
;	          combine together.  Each of the input arrays must be two 
;	          dimensional and must have the same dimensions.  These arrays
;	          should then be stacked together into a single 3-D array,
;	          creating INARR.
; OUTPUTS:
;	outarr -- The output array.  It will have dimensions equal to the
;	          first two dimensions of the input array.
; RESTRICTIONS:
;	This procedure is *SLOW* because it must loop over each pixel of the
;	image.   See notes below about an alternative with CALL_EXTERNAL.
;
; EXAMPLE:
;	Suppose one wants to combine three floating point 1024 x 1024 bias 
;	frames which have been read into the IDL variables im1,im2,im3
;
;	IDL> bigim = fltarr(1024,1024,3)	;Create big array to hold images
;	IDL> bigim(0,0,0) = im1 & bigim(0,0,1) = im2 & bigim(0,0,2) = im2  
;	IDL> medarr, bigim, avgbias
;
;	The variable avgbias will be the desired 1024x 1024 float image.
; PROCEDURE:
;	A scalar median function over the third dimension is looped over 
;       each pixel of the first two dimensions.    For an even number of image
;	the IDL MEDIAN() function cannot be used since it is biased toward
;	high values, e.g. it selects the third of four sorted numbers.  Instead
;	the pixels are explicitly sorted, and the average of the two middle
;	pixels is taken.
;
;       MEDARR is also available as a C procedure linked to IDL via
;       CALL_EXTERNAL.   The callable C version is 2-3 times faster for large 
;	(~ 500 x 500 x 7) images.   Contact W. Landsman 
;	(landsman@stars.gsfc.nasa.gov) for the C program
; MODIFICATION HISTORY:
;	Written by Michael R. Greason, STX, 12 June 1990.
;	Don't use MEDIAN function for even number of images W. Landsman Sep 1996
;-
 On_error,2
;			Check parameters.

 if N_params() LT 2 then begin			; # parameters.
	print, "Syntax -  MEDARR, inputarr, outputarr"
	return
 endif
 
 s = size(inarr)
 if s(0) NE 3 then $			; Input array size.
	message, "Input array must have 3 dimensions"

;			Create the output array.
 ncol = s(1)
 nrow = s(2)
 narr = s(3)
 type = s(s(0) + 1)
 outarr = make_array( dimen = [ncol,nrow], /NOZERO, TYPE = type )
 even = (narr mod 2) EQ 0
 if even then io = (narr-1)/2

;			Combine the input arrays into the output array.

 for j = 0, (nrow-1) do begin    
       	for i = 0, (ncol-1) do begin   
		marr = inarr(i,j,*)    ;Scalar median over 3rd dim.
		if even then begin
		marr = marr( sort(marr) )
		outarr(i,j) = 0.5*( marr(io) + marr(io+1) )		
		endif else outarr(i,j) = median(marr)       
	endfor
 endfor

 return
 end
